/**
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE":
 * The Cocoakit author wrote this file. As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.
 * ----------------------------------------------------------------------------
 */

#import "CKViewController.h"

#import "CKCalloutView.h"

/**
 * Defines a NIB file name
 */
#define NIB_FILE_NAME                                                   @"CKViewController"

@interface CKViewController ()

#pragma mark - Declared private methods

/**
 * Release all graphic elements
 *
 * @private
 */
- (void)releaseCKViewControllerGraphicElements;

@end

@implementation CKViewController

#pragma mark - Memory management

/**
 * Deallocates the memory occupied by the receiver.
 */
- (void)dealloc {

    [self releaseCKViewControllerGraphicElements];
    
    [super dealloc];
    
}

#pragma mark - View lifecycle

/**
 * Called after the controller’s view is loaded into memory.
 */
- (void)viewDidLoad {
    
    [super viewDidLoad];
    
    if (calloutView_ == nil) {
        
        calloutView_ = [[CKCalloutView alloc] initWithFrame:CGRectMake(20, 20, 280, 120)];
        
    }
    
    [[self view] addSubview:calloutView_];
    
}

/**
 * Called when the controller’s view is released from memory.
 */
- (void)viewDidUnload {
    
    [self releaseCKViewControllerGraphicElements];
    
    [super viewDidUnload];
    
}

/**
 * Returns a Boolean value indicating whether the view controller supports the specified orientation.
 *
 * @param interfaceOrientation: The orientation of the application’s user interface after the rotation.
 * @return YES if the view controller auto-rotates its view to the specified orientation; otherwise, NO .
 */
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    
    // Return YES for supported orientations
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    
}

#pragma mark - Instance initialization

/*
 * Creates and return an autoreleased CKViewController constructed from NIB file.
 */
+ (CKViewController *)ckViewController {
    
    return [[[CKViewController alloc] initWithNibName:NIB_FILE_NAME bundle:nil] autorelease];
    
}

#pragma mark - Private methods

/*
 * Release all graphic elements
 */
- (void)releaseCKViewControllerGraphicElements {
    
    [calloutView_ release];
    calloutView_ = nil;
    
}

@end
