/**
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE":
 * The Cocoakit author wrote this file. As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.
 * ----------------------------------------------------------------------------
 */

#import "CKCalloutView.h"

/**
 * Defines a widht of arrow callout
 */
#define ARROW_CALLOUT_WIDTH                                             10.0f

/**
 * Defines a half widht of arrow callout
 */
#define ARROW_CALLOUT_WIDTH_2                                           (ARROW_CALLOUT_WIDTH / 2)

/**
 * Defines a height of arrow callout
 */
#define ARROW_CALLOUT_HEIGHT                                            10.0f

/**
 * Defines a half height of arrow callout
 */
#define ARROW_CALLOUT_HEIGHT_2                                          (ARROW_CALLOUT_HEIGHT / 2)

/**
 * Defines a margint to draw callout
 */
#define CALLOUT_MARGIN                                                  5.0f;

@implementation CKCalloutView

#pragma mark - Memory management

/**
 * Deallocates the memory occupied by the receiver.
 */
- (void)dealloc {
    
    
    [super dealloc];
    
}

#pragma mark - Instance initialization

/**
 * Initializes and returns a newly allocated view object with the specified frame rectangle.
 *
 * @param aRect: The frame rectangle for the view, measured in points.
 * @return An initialized view object or nil if the object couldn't be created.
 */
- (id)initWithFrame:(CGRect)frame {
    
    self = [super initWithFrame:frame];
    
    if (self) {
        
        [self setBackgroundColor:[UIColor clearColor]];
        
    }
    
    return self;
    
}

#pragma mark - Overwrite draw methods

/**
 * Draws the receiver’s image within the passed-in rectangle.
 * 
 * @param rect: The portion of the view’s bounds that needs to be updated.
 */
- (void)drawRect:(CGRect)rect {
 
    CGContextRef context = UIGraphicsGetCurrentContext();
    
    CGFloat minX = CGRectGetMinX(rect) + CALLOUT_MARGIN;
    CGFloat minY = CGRectGetMinY(rect) + CALLOUT_MARGIN;
    CGFloat maxX = CGRectGetMaxX(rect) - CALLOUT_MARGIN;
    CGFloat maxY = CGRectGetMaxY(rect) - CALLOUT_MARGIN;
    CGFloat midX = CGRectGetMidX(rect);
    CGFloat midY = CGRectGetMidY(rect);
    
    CGMutablePathRef path = CGPathCreateMutable();
 
    CGContextSaveGState(context);
    {
        // Create path
        CGPathMoveToPoint(path, NULL, minX, minY);
        CGPathAddLineToPoint(path, NULL, maxX, minY);
        CGPathAddLineToPoint(path, NULL, maxX, maxY - ARROW_CALLOUT_HEIGHT);
        CGPathAddLineToPoint(path, NULL, midX + ARROW_CALLOUT_WIDTH_2, maxY - ARROW_CALLOUT_HEIGHT);
        CGPathAddLineToPoint(path, NULL, midX, maxY);
        CGPathAddLineToPoint(path, NULL, midX - ARROW_CALLOUT_WIDTH_2, maxY - ARROW_CALLOUT_HEIGHT);
        CGPathAddLineToPoint(path, NULL, minX, maxY - ARROW_CALLOUT_HEIGHT);
        CGPathCloseSubpath(path);
        
        // Move path to origin x,y
        CGContextTranslateCTM(context, CGRectGetMinX(rect), CGRectGetMinY(rect));
        
        // Fill path to color
        CGContextSaveGState(context);
        {
            
            CGContextClip(context);
            CGContextAddPath(context, path);
            CGContextSetFillColorWithColor(context, [UIColor whiteColor].CGColor);
            CGContextFillPath(context);
            
        }
        CGContextRestoreGState(context);
        
        // Stroke path to color
        CGContextSaveGState(context);
        {
            
            CGContextClip(context);
            CGContextAddPath(context, path);
            CGContextSetLineWidth(context, 4.0f);
            CGContextSetStrokeColorWithColor(context, [UIColor blackColor].CGColor);
            CGContextStrokePath(context);
            
        }
        CGContextRestoreGState(context);
        
        // Add text
        CGContextSaveGState(context);
        {
            
            UIFont *font = [UIFont boldSystemFontOfSize:16.0f];
            NSString *text = @"Hi Callout!";
            CGSize sizeFont = [text sizeWithFont:font];
            
            [text drawInRect:CGRectMake(midX - (sizeFont.width / 2), midY - (sizeFont.height / 2) - ARROW_CALLOUT_HEIGHT_2, sizeFont.width, sizeFont.height) withFont:font];
            
        }
        CGContextRestoreGState(context);
        
    }
    CGContextRestoreGState(context);
    
    CGPathRelease(path);
 
}

@end
